////////////////////////////////////////////////////////////////////////
// Class:       MonitorLiveTime
// Module Type: analyzer
// File:        MonitorLiveTime_module.cc
//
// Generated at Tue Apr  9 14:28:00 2013 by Nicola Rossi using artmod
// from art v1_03_08.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "artdaq-core/Data/Fragments.hh"
#include "darksidecore/Data/V1495Fragment.hh"
#include "darksidecore/ArtServices/DBInterface.hh"

#include <iostream>
#include <iomanip>

namespace ds50 {
  class MonitorLiveTime : public art::EDAnalyzer {
    public:
      explicit MonitorLiveTime(fhicl::ParameterSet const & p);
      virtual ~MonitorLiveTime();

      void beginRun(art::Run const &) override;
      void analyze(art::Event const & e) override;
      void endRun(art::Run const &) override;

    private:
      time_t start_time_, stop_time_;
      suseconds_t start_time_us_, stop_time_us_;
      double live_time_v1495_, dead_time_v1495_;
      V1495Fragment::count_t gps_coarse_s_, gps_fine_20ns_, gps_pps_20ns_;
      long events_, invalid_, overflow_;
  };
}

ds50::MonitorLiveTime::MonitorLiveTime(fhicl::ParameterSet const &p) :
  art::EDAnalyzer(p) { }

ds50::MonitorLiveTime::~MonitorLiveTime() { }

void ds50::MonitorLiveTime::beginRun(art::Run const &) {
  start_time_ = 0;
  stop_time_ = 0;
  live_time_v1495_ = 0;
  dead_time_v1495_ = 0;
  events_ = 0;
  invalid_ = 0;
  overflow_ = 0;
  gps_coarse_s_ = 0;
}

void ds50::MonitorLiveTime::analyze(art::Event const & e) {
  art::Handle<artdaq::Fragments> v1495;
  e.getByLabel ("daq", "V1495", v1495);
  if (!v1495.isValid ()) {
    invalid_++;
    return;
  }

  if(v1495->size() != 1) {
    mf::LogError ("MonitorLiveTime") << v1495->size() << " fragment for V1495";
    invalid_++;
    return; 
  }

  if ((*v1495)[0].hasMetadata ()) {
    V1495Fragment b((*v1495)[0]);
    V1495Fragment::metadata const* md = (*v1495)[0].metadata<V1495Fragment::metadata> ();

    time_t trigger_time =  md->trigger_time.tv_sec;
    suseconds_t trigger_time_us = md->trigger_time.tv_usec;
    if (!start_time_) { 
      start_time_ = trigger_time;
      start_time_us_ = trigger_time_us;
    }

    if (trigger_time < stop_time_) mf::LogError ("MonitorLiveTime") << "time is flowing back " << difftime (trigger_time, stop_time_) << " s at event " << b.trigger_counter ();
    else {
      stop_time_ = trigger_time;
      stop_time_us_ = trigger_time_us;
    }

    if ((b.live_time_100ns () == 0xFFFFFFFF) || (b.incremental_inhibit_time_100ns () == 0xFFFFFFFF)) overflow_++;

    live_time_v1495_ += b.live_time_100ns () * 100e-9;
    dead_time_v1495_ += b.incremental_inhibit_time_100ns () * 100e-9;

    if (gps_coarse_s_ <= b.gps_coarse_s ()) {
      gps_coarse_s_ = b.gps_coarse_s ();
      gps_fine_20ns_ = b.gps_fine_20ns ();
      gps_pps_20ns_ = b.gps_pps_20ns ();
    }

    events_++;
  } else invalid_++;
}


void ds50::MonitorLiveTime::endRun(art::Run const &e) {
  std::ostringstream start_s, stop_s;
  start_s << start_time_ << "." << std::setw(6) << std::setfill('0') << start_time_us_;
  stop_s << stop_time_ << "." << std::setw(6) << std::setfill('0') << stop_time_us_;
  double dt = difftime (stop_time_, start_time_) + (stop_time_us_ - start_time_us_) * 1e-6;
  std::cout << "run: " << e.run () << std::endl;
  std::cout << "events: " << events_ << std::endl;
  std::cout << "invalid: " << invalid_ << std::endl;
  std::cout << "overflow: " << overflow_ << std::endl;
  std::cout << "start: " << start_s.str () << std::endl;
  std::cout << "stop: " << stop_s.str () << std::endl;
  std::cout << "runtime DAQ: " << dt << std::endl;
  std::cout << "runtime GPS: " << double(gps_coarse_s_) + gps_fine_20ns_ * 20e-9 << std::endl;
  std::cout << "livetime v1495: " << live_time_v1495_ << std::endl;
  std::cout << "deadtime v1495: " << dead_time_v1495_ << std::endl;
  std::cout << "missing time: " << dt - live_time_v1495_ - dead_time_v1495_ << std::endl;

  try {
    art::ServiceHandle<ds50::DBInterface> dbi;
    ds50::db::result r = dbi->latest ("shadowside.livetime", "");
    r.wipe ();
    r.set ("version", 0, "MonitorLiveTime");
    r.set ("run", 0, e.run ());
    r.set ("events", 0, events_);
    r.set ("start", 0, dbi->query ("select to_timestamp(" + start_s.str () + ")").get<std::string> (0,0));
    r.set ("stop", 0, dbi->query ("select to_timestamp(" + stop_s.str () + ")").get<std::string> (0,0));
    r.set ("runtime_daq", 0, dt);
    r.set ("runtime_gps", 0, double(gps_coarse_s_) + gps_fine_20ns_ * 20e-9);
    r.set ("livetime_v1495", 0, live_time_v1495_);
    r.set ("deadtime_v1495", 0, dead_time_v1495_);

    dbi->insert (r, "MonitorLiveTime");
  } catch (const cet::exception& e) {
    mf::LogError ("MonitorLiveTime") << "exception: " << e.what ();
  }
}

DEFINE_ART_MODULE(ds50::MonitorLiveTime)
